"use strict";

var _path = require("path");

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _traverse2 = _interopRequireDefault(require("./traverse"));

var _VirtualModulePlugin = _interopRequireDefault(require("./VirtualModulePlugin"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

// can'ts use class syntax b/c babel doesn't transpile it correctly for Error
function CssLiteralLoaderError(error) {
  Error.call(this);
  this.name = 'CssLiteralLoaderError';
  this.message = error.message;
  if (error.codeFrame) this.message += "\n\n " + error.codeFrame + " \n";
  this.error = error;

  try {
    this.stack = error.stack.replace(/^(.*?):/, this.name + ":");
  } catch (err) {
    Error.captureStackTrace(this, CssLiteralLoaderError);
  }
}

CssLiteralLoaderError.prototype = Object.create(Error.prototype);
CssLiteralLoaderError.prototype.constructor = CssLiteralLoaderError;

function collectStyles(src, filename, opts) {
  // quick regex as an optimization to avoid parsing each file
  if (!src.match(new RegExp(opts.tagName + "`([\\s\\S]*?)`", 'gmi'))) {
    return {
      styles: []
    };
  } // maybe eventually return the ast directly if babel-loader supports it


  try {
    var _traverse = (0, _traverse2.default)(src, filename, _extends({}, opts, {
      writeFiles: false
    })),
        metadata = _traverse.metadata;

    return {
      styles: metadata['css-literal-loader'].styles || []
    };
  } catch (err) {
    throw new CssLiteralLoaderError(err);
  }
}

function replaceStyleTemplates(src, styles) {
  var offset = 0;

  function splice(str, start, end, replace) {
    var result = str.slice(0, start + offset) + replace + str.slice(end + offset);
    offset += replace.length - (end - start);
    return result;
  }

  styles.forEach(function (_ref) {
    var start = _ref.start,
        end = _ref.end,
        path = _ref.path;
    src = splice(src, start, end, "require('./" + (0, _path.basename)(path) + "')");
  });
  return src;
}

var LOADER_PLUGIN = Symbol('loader added VM plugin');

module.exports = function loader(content) {
  if (this.cacheable) this.cacheable();
  var options = _loaderUtils.default.getOptions(this) || {};
  var tagName = options.tagName,
      _options$extension = options.extension,
      extension = _options$extension === void 0 ? '.css' : _options$extension;

  var _collectStyles = collectStyles(content, this.resourcePath, {
    tagName: tagName,
    extension: extension
  }),
      styles = _collectStyles.styles;

  if (!styles.length) return content;
  var emitVirtualFile = this.emitVirtualFile; // The plugin isn't loaded

  if (!emitVirtualFile) {
    var compilation = this._compilation; // eslint-disable-line no-underscore-dangle

    var plugin = compilation[LOADER_PLUGIN];

    if (!plugin) {
      plugin = _VirtualModulePlugin.default.bootstrap(compilation);
      compilation[LOADER_PLUGIN] = plugin;
    }

    emitVirtualFile = plugin.addFile;
  }

  styles.forEach(function (style) {
    // style.path = `${basepath}__css_literal_loader_${idx++}${extension}`;
    emitVirtualFile(style.path, style.value);
  });
  return replaceStyleTemplates(content, styles);
};