"use strict";

exports.__esModule = true;
exports.default = plugin;

var _fs = require("fs");

var _path = require("path");

var t = _interopRequireWildcard(require("@babel/types"));

var _template = _interopRequireDefault(require("@babel/template"));

var _commonTags = require("common-tags");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

var buildImport = (0, _template.default)('require(FILENAME);');
var STYLES = Symbol('CSSLiteralLoader');
var idx = 0;

function createFilename(hostFile, _ref) {
  var _ref$extension = _ref.extension,
      extension = _ref$extension === void 0 ? '.css' : _ref$extension;
  var basepath = (0, _path.join)((0, _path.dirname)(hostFile), (0, _path.basename)(hostFile, (0, _path.extname)(hostFile)));
  return basepath + "__" + idx++ + "_extracted_style" + extension;
}

function plugin() {
  return {
    pre: function pre(file) {
      if (!file.has(STYLES)) {
        file.set(STYLES, new Set());
      }
    },
    post: function post(file) {
      var opts = this.opts;
      var styles = file.get(STYLES);
      styles = Array.from(styles.values());
      file.metadata['css-literal-loader'] = {
        styles: styles
      };

      if (opts.writeFiles !== false) {
        styles.forEach(function (_ref2) {
          var path = _ref2.path,
              value = _ref2.value;
          (0, _fs.writeFileSync)(path, (0, _commonTags.stripIndent)([value]));
        });
      }
    },
    visitor: {
      TaggedTemplateExpression: function TaggedTemplateExpression(path, _ref3) {
        var opts = _ref3.opts,
            file = _ref3.file;
        var styles = file.get(STYLES);
        var _opts$tagName = opts.tagName,
            tagName = _opts$tagName === void 0 ? 'css' : _opts$tagName;
        var node = path.node;

        if (node.tag.name !== tagName || !path.scope.hasGlobal(tagName)) {
          return;
        }

        var parseError = path.buildCodeFrameError('Could not evaluate css. Inline css must be statically analyzable');
        var start = node.start,
            end = node.end; // remove the tag and evaluate as a plain template;

        path.replaceWith(node.quasi);

        var _path$evaluate = path.evaluate(),
            confident = _path$evaluate.confident,
            value = _path$evaluate.value;

        if (!confident) {
          throw parseError;
        }

        var style = {
          value: value,
          start: start,
          end: end
        };
        var getFilename = opts.getFilename || createFilename;
        var hostFile = file.opts.filename;
        style.path = getFilename(hostFile, opts);
        var filename = (0, _path.relative)((0, _path.dirname)(hostFile), style.path);

        if (!filename.startsWith('.')) {
          filename = "./" + filename;
        }

        path.replaceWith(buildImport({
          FILENAME: t.StringLiteral(filename)
        }) // eslint-disable-line new-cap
        );
        styles.add(style);
      }
    }
  };
}